using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

using YD = Yanesdk.Draw;

namespace CornerCrash
{
    class BGLayer : BaseLayer
    {
        BGMap BG;
        Bitmap BGBitmap;
        YD.Screen2DGl screen;
        YD.GlTexture BGTexture;
        YD.DrawContext MapContext;

        public void Init(BGMap pBG, YD.Screen2DGl pscreen)
        {
            Init(pBG, null);
        }

        public void Init(BGMap pBG, Character character, YD.Screen2DGl pscreen)
        {
            BG = pBG;
            baseChar = character;
            screen = pscreen;
            MapContext = new Yanesdk.Draw.DrawContext();
            MapContext.SetScreenSize(CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT);
            MapContext.SetClipRect(new YD.Rect(0,0,CornerCrashDefine.WINDOW_WIDTH,
                CornerCrashDefine.WINDOW_HEIGHT));
            MapContext.SetOffset(0, 0);

            base.Init(BG.Map[0].Length * BGMap.MAPCHIP_WIDTH, BG.Map.Length * BGMap.MAPCHIP_HEIGHT);
        }

        private void DrawBGMap()
        {
            int maph = BG.Map.Length;
            int mapw = BG.Map[0].Length;

            if ((GameState.getInstance().State == GameState.Mode.TITLE) || 
                (GameState.getInstance().State == GameState.Mode.GAMEOVER) ||
                (GameState.getInstance().State == GameState.Mode.STGSTART))
            {
                return;
            }

            if (BGTexture == null)
            {
                BGTexture = new Yanesdk.Draw.GlTexture();
                BGBitmap = new Bitmap(mapw * BGMap.MAPCHIP_WIDTH, maph * BGMap.MAPCHIP_HEIGHT);
                Graphics g = Graphics.FromImage(BGBitmap);
                
                for (int i = 0; i < maph; i++)
                {
                    for (int j = 0; j < mapw; j++)
                    {
                        if ((BG.Map[i][j] >= BG.Part.Length) || (BG.Map[i][j] < 0))
                        {
                            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                                "}bv`bv̒l " + BG.Map[i][j] + "͔͈͊OłB");
                        }

                        BG.Part[BG.Map[i][j]].x = (j * BGMap.MAPCHIP_WIDTH);
                        BG.Part[BG.Map[i][j]].y = (i * BGMap.MAPCHIP_HEIGHT);
                        g.DrawImage(BG.Part[BG.Map[i][j]].Bmp, BG.Part[BG.Map[i][j]].x, BG.Part[BG.Map[i][j]].y,
                            BGMap.MAPCHIP_WIDTH, BGMap.MAPCHIP_HEIGHT);
                        //g.DrawImage(BG.Part[BG.Map[i][j]].Bmp, BG.Part[BG.Map[i][j]].x, BG.Part[BG.Map[i][j]].y);
                    }
                }
                BGTexture.SetBitmap(BGBitmap);
                BGBitmap.Dispose();
                BGBitmap = null;
                g.Dispose();
            }
            if (basePointChar != null)
            {
                CnvCharPointToBasePoint();
            }

            screen.Blt(BGTexture, -basex, -basey);
        }

        public void setRealPoint(BGPart bg)
        {
            if (basePointChar != null)
            {
                CnvCharPointToBasePoint();
            }
            bg.sprite.x = bg.x;
            bg.sprite.y = bg.y;
        }

        public override void DrawLayer()
        {
            DrawBGMap();
        }

        private void TransBGMap()
        {
            if (basePointChar != null)
            {
                CnvCharPointToBasePoint();
            }
            
        }

        public override LayerType Type
        {
            get
            {
                return LayerType.BG;
            }
        }

        public override void Dispose()
        {
            base.Dispose();
            BG.Dispose();
            MapContext = null;
            if (BGTexture != null)
            {
                if (BGTexture.Loaded == true)
                {
                    BGTexture.Release();
                }
                BGTexture = null;
            }
        }
    }
}
