using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

using YD = Yanesdk.Draw;

namespace CornerCrash
{
    class BlindLayer : BaseLayer
    {
        YD.Screen2DGl screen;
        int maskw;
        int maskh;
        const string imagepath = "bgimg/mask.png";
        YD.GlTexture maskTexture;
        int framecnt;
        int px, py;

        public BlindLayer(YD.Screen2DGl pscreen, List<Character> chars)
        {
            screen = pscreen;
            characters = chars;
            maskTexture = new Yanesdk.Draw.GlTexture();

            maskw = (BGMap.MAPCHIP_WIDTH * 5);
            maskh = (BGMap.MAPCHIP_HEIGHT * 5);
            framecnt = 6;
        }

        public void InitPos()
        {
            if (basePointChar == null)
            {
                foreach (Character charp in characters)
                {
                    if (charp.CharType == CornerCrashDefine.characterType.Player)
                    {
                        basePointChar = charp;
                    }
                }
            }
            px = (basePointChar.sprite.x - (maskw / 2));
            py = (basePointChar.sprite.y - (maskh / 2));
        }

        public override void DrawLayer()
        {
            double pxend = 0;
            double pyend = 0;

            if ((GameState.getInstance().State == GameState.Mode.TITLE) ||
                (GameState.getInstance().State == GameState.Mode.GAMEOVER) ||
                (GameState.getInstance().State == GameState.Mode.STGSTART))
            {
                return;
            }

            if (basePointChar == null)
            {
                foreach (Character charp in characters)
                {
                    if (charp.CharType == CornerCrashDefine.characterType.Player)
                    {
                        basePointChar = charp;
                    }
                }
                px = (basePointChar.sprite.x - (maskw / 2));
                py = (basePointChar.sprite.y - (maskh / 2));
            }

            if (basePointChar == null)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "vC[LN^[XgɌ܂łB");
            }

            if (maskTexture.Loaded == false)
            {
                maskTexture.Load(imagepath);
            }
            int r, g, b, a;

            screen.GetColor(out r, out g, out b, out a);
            screen.SetColor(0, 0, 0, 240);

            pxend = (double)(basePointChar.sprite.x - (maskw / 2));
            pyend = (double)(basePointChar.sprite.y - (maskh / 2));

            switch (basePointChar.NowDirect)
            {
                case Character.NowDirection.TOP:
                    pyend -= maskh / 3;
                    break;
                case Character.NowDirection.BOTTOM:
                    pyend += maskh / 3;
                    break;
                case Character.NowDirection.LEFT:
                    pxend -= maskw / 3;
                    break;
                case Character.NowDirection.RIGHT:
                    pxend += maskw / 3;
                    break;
            }

            //double length = 0;

            if ((Math.Abs((int)pxend - px) > (Math.Abs(pxend - (double)px) / (double)framecnt)) &&
                (Math.Abs((int)pyend - py) > (Math.Abs(pyend - (double)py) / (double)framecnt)))
            {
//                length = Math.Sqrt(Math.Pow((double)pxend - (double)px, 2.0) + Math.Pow((double)pyend - (double)py, 2.0));
//                px += (int)((Math.Abs(pxend - (double)px) / (pxend - (double)px)) * Math.Abs((pxend - (double)px) / (length / (double)move)));
//                py += (int)((Math.Abs(pyend - (double)py) / (pyend - (double)py)) * Math.Abs((pyend - (double)py) / (length / (double)move)));
                px += (int)((Math.Abs(pxend - (double)px) / (pxend - (double)px)) * Math.Abs((pxend - (double)px) / (double)framecnt));
                py += (int)((Math.Abs(pyend - (double)py) / (pyend - (double)py)) * Math.Abs((pyend - (double)py) / (double)framecnt));
            }
            else if (Math.Abs((int)pxend - px) > (Math.Abs(pxend - (double)px) / (double)framecnt))
            {
//                px += (int)((Math.Abs(pxend - (double)px) / (pxend - (double)px)) * (double)move);
                px += (int)((Math.Abs(pxend - (double)px) / (pxend - (double)px)) * ((pxend - (double)px) / (double)framecnt));
                py = (int)pyend;
            }
            else if (Math.Abs((int)pyend - py) > (Math.Abs(pyend - (double)py) / (double)framecnt))
            {
//                py += (int)((Math.Abs(pyend - (double)py) / (pyend - (double)py)) * (double)move);
                py += (int)((Math.Abs(pyend - (double)py) / (pyend - (double)py)) * ((pyend - (double)py) / (double)framecnt));
                px = (int)pxend;
            }
            else
            {
                px = (int)pxend;
                py = (int)pyend;
            }

            foreach (Character charp in characters)
            {
                if (charp.CharType == CornerCrashDefine.characterType.Player)
                {
                    continue;
                }

                if (charp.chkIsinRealBoxArea(px, py, maskw, maskh) == false)
                {
                    charp.sprite.enable = false;
                }
                else
                {
                    charp.sprite.enable = true;
                }
            }

            if ((px > 0) && (py > 0))
            {
                screen.DrawPolygon(0, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                    CornerCrashDefine.WINDOW_WIDTH, py, 0, py);
                screen.DrawPolygon(0, py, px, py, px, CornerCrashDefine.WINDOW_HEIGHT,
                    0, CornerCrashDefine.WINDOW_HEIGHT);
                screen.DrawPolygon(px, py + maskh, px + maskw, py + maskh,
                    px + maskw, CornerCrashDefine.WINDOW_HEIGHT, px, CornerCrashDefine.WINDOW_HEIGHT);
                screen.DrawPolygon(px + maskw, py, px + CornerCrashDefine.WINDOW_WIDTH, py,
                    CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                    px + maskw, CornerCrashDefine.WINDOW_HEIGHT );
            }
            else if (px > 0)
            {
                screen.DrawPolygon(0, 0, px, 0, px, py + maskh, 0, py + maskh);
                screen.DrawPolygon(0, py + maskh, CornerCrashDefine.WINDOW_WIDTH, py + maskh,
                    CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                    0, CornerCrashDefine.WINDOW_HEIGHT);
                screen.DrawPolygon(px + maskw, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                    CornerCrashDefine.WINDOW_WIDTH, py + maskh, px + maskw, py + maskh);
            }
            else if (py > 0)
            {
                screen.DrawPolygon(0, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                    CornerCrashDefine.WINDOW_WIDTH, py, 0, py);
                screen.DrawPolygon(px + maskw, py, CornerCrashDefine.WINDOW_WIDTH, py,
                    CornerCrashDefine.WINDOW_WIDTH, py + maskh, px + maskw, py + maskh);
                screen.DrawPolygon(0, py + maskh, CornerCrashDefine.WINDOW_WIDTH, py + maskh,
                    CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                    0, CornerCrashDefine.WINDOW_HEIGHT);
            }
            else
            {
                screen.DrawPolygon(px + maskw, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                    CornerCrashDefine.WINDOW_WIDTH, py + maskh,
                    px + maskw, py + maskh);
                screen.DrawPolygon(0, py + maskh, CornerCrashDefine.WINDOW_WIDTH, py + maskh,
                    CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                    0, CornerCrashDefine.WINDOW_HEIGHT);
            }
            screen.Blt(maskTexture, px, py);
            screen.SetColor(r, g, b, a);
        }

        public override LayerType Type
        {
            get
            {
                return LayerType.BLIND;
            }
        }

    }
}
