using System;
using System.Collections.Generic;
using System.Text;

using YD = Yanesdk.Draw;

namespace CornerCrash
{
    class Character : IDisposable
    {
        public enum NowDirection
        {
            VOID,
            TOP,
            BOTTOM,
            LEFT,
            RIGHT,
            NOTHING,
        }

        protected NowDirection[] NextDirectTbl = {
            NowDirection.VOID,
            NowDirection.TOP,
            NowDirection.BOTTOM,
            NowDirection.LEFT,
            NowDirection.RIGHT,
        };

        protected enum BackMoveSt
        {
            NoMove = 0,
            MoveR,
            MoveL,
        }

        protected BackMoveSt backmvstate;
        protected bool isvalid;

        protected int beforeDirection;
        protected YD.GlTexture texture;
        protected YD.Screen2DGl screen;

        public BaseSprite sprite;
        protected BGMap bgmap;
        protected List<Character> characters;

        protected int mx, my;
        public int x, y;
        protected NowDirection nowdirect;
        protected TurnDirection turndirect;
        public bool enable;
        protected int animinterval;
//        protected const int animintervalbase = 5;
        protected const int animintervalbase = 32;
        protected int animintervalmax;

        protected const string icondir = "charimg/icon";

        public Character()
        {

        }

        public Character(YD.Screen2DGl pscreen, BGMap pMap)
        {
            isvalid = true;
            screen = pscreen;
            bgmap = pMap;
            x = 0;
            y = 0;
            mx = 8;
            my = 8;
            LoadSprite();
            turndirect = new TurnDirection();
            turndirect.setupTable(this, bgmap);
            enable = true;
            animintervalmax = animintervalbase;
            animinterval = animintervalmax;
        }

        public Character(YD.Screen2DGl pscreen, BGMap pMap, List<Character> pchars)
        {
            isvalid = true;
            screen = pscreen;
            bgmap = pMap;
            characters = pchars;
            x = 0;
            y = 0;
            mx = 4;
            my = 4;
            LoadSprite();
            turndirect = new TurnDirection();
            turndirect.setupTable(this, bgmap);
            animinterval = animintervalmax;
        }

        public virtual void LoadSprite()
        {
            texture = new Yanesdk.Draw.GlTexture();
            sprite = new BaseSprite(screen, texture);
        }

        public virtual void Control()
        {

        }

        public virtual void setMapPosition(int sx, int sy)
        {

        }

        public virtual void setCharacters(List<Character> pchars)
        {
            characters = pchars;
        }

        protected virtual bool IsXMove(bool isadd)
        {
            int chkpnt = 0;
            int bgx = 0;

            if (isadd == false)
            {
                chkpnt = x - mx;

                if (chkpnt < 0)
                {
                    return false;
                }

                bgx = chkpnt / BGMap.MAPCHIP_WIDTH;
            }
            else
            {
                chkpnt = x + sprite.width + mx;
                if ((chkpnt / BGMap.MAPCHIP_WIDTH) >= bgmap.Map[0].Length)
                {
                    return false;
                }

                bgx = chkpnt / BGMap.MAPCHIP_WIDTH;
            }

            int loopend = ((y + sprite.height) / BGMap.MAPCHIP_HEIGHT);
            for (int i = ((y + sprite.height / 2) / BGMap.MAPCHIP_HEIGHT); i <= loopend; i++)
            {
                if (bgmap.BGTypes[bgmap.Map[i][bgx]] == BGMap.BGType.BLOCK)
                {
                    return false;
                }
            }
            return true;
        }

        protected virtual bool IsYMove(bool isadd)
        {
            int chkpnt = 0;
            int bgy = 0;

            if (isadd == false)
            {
                chkpnt = y - my;

                if (chkpnt < 0)
                {
                    return false;
                }

                bgy = chkpnt / BGMap.MAPCHIP_HEIGHT;
            }
            else
            {
                chkpnt = y + sprite.height + my;

                if ((chkpnt / BGMap.MAPCHIP_HEIGHT) >= bgmap.Map.Length)
                {
                    return false;
                }

                bgy = chkpnt / BGMap.MAPCHIP_HEIGHT;
            }

            chkpnt += sprite.height / 2;

            int loopend = ((x + sprite.width) / BGMap.MAPCHIP_WIDTH);
            for (int i = (x / BGMap.MAPCHIP_WIDTH); i <= loopend; i++)
            {
                if (bgmap.BGTypes[bgmap.Map[bgy][i]] == BGMap.BGType.BLOCK)
                {
                    return false;
                }
            }
            return true;
        }

        public bool chkIsinBoxArea(int sx, int sy, int w, int h)
        {
            if (
                (
                  ((x >= sx) && (x <= sx + w)) ||
                  (((x + sprite.width) >= sx) && ((x + sprite.width) <= sx + w))
                ) && (
                  ((y >= sy) && (y <= sy + h)) ||
                  (((y + sprite.height) >= sy) && ((y + sprite.height) <= sy + h))
                )
              )
            {
                return true;
            }
            return false;
        }

        public bool chkIsinRealBoxArea(int sx, int sy, int w, int h)
        {
            if (
                (
                  ((sprite.x >= sx) && (sprite.x <= sx + w)) ||
                  (((sprite.x + sprite.width) >= sx) && ((sprite.x + sprite.width) <= sx + w))
                ) && (
                  ((sprite.y >= sy) && (sprite.y <= sy + h)) ||
                  (((sprite.y + sprite.height) >= sy) && ((sprite.y + sprite.height) <= sy + h))
                )
              )
            {
                return true;
            }
            return false;
        }

        public virtual bool chkIsinBoxAreaNextPos(int chkx, int chky)
        {
            for (int i = 0; i < characters.Count; i++)
            {
                if (characters[i] == this)
                {
                    continue;
                }

                if (characters[i].chkIsinBoxArea(chkx, chky, sprite.width, sprite.height) == true)
                {
                    return true;
                }
            }
            return false;
        }

        protected virtual bool IsCorner()
        {

            turndirect.setupTable(this, bgmap);
            return turndirect.IsCorner();
        }

        protected NowDirection getBackDirection(NowDirection direct)
        {
            switch (direct)
            {
                case NowDirection.TOP:
                    return NowDirection.BOTTOM;
                case NowDirection.BOTTOM:
                    return NowDirection.TOP;
                case NowDirection.LEFT:
                    return NowDirection.RIGHT;
                case NowDirection.RIGHT:
                    return NowDirection.LEFT;
                default:
                    return NowDirection.VOID;
            }
        }

        public virtual void addGamePoint(int point)
        {
            
        }

        public virtual void negGamePoint(int point)
        {

        }

        public virtual void drawIcon()
        {

        }

        public virtual void HitofCharNotice(Character charp)
        {

        }

        protected void decAnimInterval()
        {
            animinterval -= (mx + my) / 2;
        }

        public bool IsValid
        {
            get
            {
                return isvalid;
            }
        }

        public virtual CornerCrashDefine.characterType CharType
        {
            get
            {
                return CornerCrashDefine.characterType.VOID;
            }
        }

        public virtual CornerCrashDefine.characterType UniqCharType
        {
            get
            {
                return CharType;
            }
        }

        public NowDirection NowDirect
        {
            get
            {
                return nowdirect;
            }
        }

        public virtual BGMap.BGType CurPosOfBGType
        {
            get
            {
                if((x / BGMap.MAPCHIP_WIDTH) != ((x + sprite.width) / BGMap.MAPCHIP_WIDTH))
                {
                    return BGMap.BGType.OVERLAP;
                }

                if((y / BGMap.MAPCHIP_HEIGHT) != ((y + sprite.height) / BGMap.MAPCHIP_HEIGHT))
                {
                    return BGMap.BGType.OVERLAP;
                }

                return bgmap.BGTypes[bgmap.Map[y / BGMap.MAPCHIP_HEIGHT][x / BGMap.MAPCHIP_WIDTH]];
            }
        }

        protected NowDirection NextMapDirect
        {

            get
            {
                if ((x / BGMap.MAPCHIP_WIDTH) != ((x + sprite.width) / BGMap.MAPCHIP_WIDTH))
                {
                    return NowDirection.VOID;
                }

                if ((y / BGMap.MAPCHIP_HEIGHT) != ((y + sprite.height) / BGMap.MAPCHIP_HEIGHT))
                {
                    return NowDirection.VOID;
                }

                BGMap.Direct direct = bgmap.getBGDirects(
                    bgmap.DirectMap[y / BGMap.MAPCHIP_HEIGHT][x / BGMap.MAPCHIP_WIDTH]);
                return NextDirectTbl[(int)direct];
            }
        }

        public virtual void Dispose()
        {
            if (texture != null)
            {
                //if (texture.Loaded == true)
                //{
                //    texture.Dispose();g܂킷̂ŃRgAEgB
                //}
                texture = null;
            }
        }
    }
}
