using System;
using System.Collections.Generic;
using System.Text;
using YD = Yanesdk.Draw;
using YS = Yanesdk.Sound;

namespace CornerCrash
{
    class Effect : IDisposable
    {
        static YD.Screen2DGl screen;
        Stat stat;
        EffectType type;
        int time;
        YS.Sound sound;

        public enum Time
        {
            FLASHEND = 10,
            BLOODEND = 60,
        }

        public enum EffectType
        {
            DEFAULT,
            BLOOD,
        }

        public enum Stat
        {
            READY,
            START,
            RUNNING,
            END,
        }

        public Effect(EffectType val)
        {
            stat = Stat.READY;
            time = 0;
            type = val;
        }

        public static void InitScreen(YD.Screen2DGl pscreen)
        {
            screen = pscreen;
        }

        public void ExecEffect()
        {
            switch (type)
            {
                case EffectType.DEFAULT:
                    FlashEffect();
                    break;
                case EffectType.BLOOD:
                    KnifeEffect();
                    break;
            }
        }

        public void FlashEffect()
        {
            if (stat == Stat.READY)
            {
                stat = Stat.RUNNING;
                sound = new YS.Sound();
                sound.Load("sound/tm2_hit004.wav");
                sound.Volume = 0.5f;
                sound.Play();
            }

            time++;

            if ((time % 2) == 0)
            {
                int r, g, b, a;

                screen.GetColor(out r, out g, out b, out a);
                screen.SetColor(255, 0, 0, 128);
                screen.DrawPolygon(0, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                    CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                    0, CornerCrashDefine.WINDOW_HEIGHT);

                screen.SetColor(r, g, b, a);
            }

            if (time >= (int)Time.FLASHEND)
            {
                stat = Stat.END;
                sound.Release();
                sound = null;
            }
        }

        public void KnifeEffect()
        {
            if (stat == Stat.READY)
            {
                stat = Stat.RUNNING;
                sound = new YS.Sound();
                sound.Load("sound/tm2_hit004.wav");
                sound.Volume = 0.5f;
                sound.Play();
            }

            time++;

            int r, g, b, a;

            screen.GetColor(out r, out g, out b, out a);
            screen.SetColor(255, 0, 0, 128);
            screen.DrawPolygon(0, 0, CornerCrashDefine.WINDOW_WIDTH, 0,
                CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT,
                0, CornerCrashDefine.WINDOW_HEIGHT);

            screen.SetColor(r, g, b, a);

            if (time >= (int)Time.BLOODEND)
            {
                stat = Stat.END;
                sound.Release();
                sound = null;
            }
        }

        public Stat EffectStat
        {
            get
            {
                return stat;
            }
        }

        public void Dispose()
        {
            if (sound != null)
            {
                if (sound.Loaded == true)
                {
                    sound.Release();
                }
                sound = null;
            }
        }
    }
}
