using System;
using System.Collections.Generic;
using System.Text;

using YD = Yanesdk.Draw;
using YI = Yanesdk.Input;

namespace CornerCrash
{
    class EventExecLayer : BaseLayer
    {
        EventTbl evttable;
        DataRecord evtrec;
        string eventtype;

        YD.Screen2DGl screen;
        BGMap map;

        YD.Textures msgtexture;
        YD.GlTexture msgwintexture;
        
        int fontwidth;
        YD.FontRepository fontrp;
        YD.FontLoader fontldr;
        YD.FontRepository bigfontrp;
        YD.FontLoader bigfontldr;
        YD.FontRepository smallfontrp;
        YD.FontLoader smallfontldr;

        string dispmsg;
        int msgstart;

        bool execevtrec_comp;

        YI.Key1 key;
        Queue<string> eventqueue;
        int framecounter;
        int waittime;
        const string stageclrmsg = "Xe[WNAII";
        const string stagestartmsg = "please press the SPACE keay to start";

        public EventExecLayer(YD.Screen2DGl pscreen, BGMap pMap)
        {
            evttable = new EventTbl();

            msgtexture = new Yanesdk.Draw.Textures();
            msgwintexture = new Yanesdk.Draw.GlTexture();

            screen = pscreen;
            map = pMap;

            fontwidth = 24;
            YD.SmartFontLoader smfldr = new Yanesdk.Draw.SmartFontLoader();
            fontldr = smfldr.LoadDefFile("mem:msgothic.ttc, " + fontwidth + ", 0, 0");
            fontrp = new YD.FontRepository(delegate { return new YD.GlTexture(); });
            fontrp.SetLoader(fontldr, 0);

            smfldr = new Yanesdk.Draw.SmartFontLoader();
            bigfontldr = smfldr.LoadDefFile("mem:msgothic.ttc, " + CornerCrashDefine.BIGFONT_WIDTH + ", 0, 0");
            bigfontrp = new YD.FontRepository(delegate { return new YD.GlTexture(); });
            bigfontrp.SetLoader(bigfontldr, 0);

            smfldr = new Yanesdk.Draw.SmartFontLoader();
            smallfontldr = smfldr.LoadDefFile("mem:msgothic.ttc, " + CornerCrashDefine.SMALLFONT_WIDTH + ", 0, 0");
            smallfontrp = new YD.FontRepository(delegate { return new YD.GlTexture(); });
            smallfontrp.SetLoader(smallfontldr, 0);

            dispmsg = "";
            msgstart = 0;
            
            execevtrec_comp = false;
            
            evtrec = null;
            eventtype = "";
            eventqueue = new Queue<string>();
            key = new Yanesdk.Input.Key1();
            framecounter = 0;
            waittime = 0;
        }

        public void Init(string filename)
        {
            evttable.ReadCsv(filename);
            msgwintexture.Load("windowimg/messagewindow.png");
        }

        public void EventExecRequest(string eventid)
        {
            eventqueue.Enqueue(eventid);
            return;
        }

        public void EventExecRequest()
        {
            string id;
            if (GameState.getInstance().State != GameState.Mode.MAIN)
            {
                return;
            }

            if (eventqueue.Count == 0)
            {
                return;
            }

            id = eventqueue.Dequeue();

            waittime = 1;
            switch (id)
            {
                case GameEventMng.StagegEndID:
                    GameState.getInstance().State = GameState.Mode.STAGEEND;
                    return;
                case GameEventMng.GameOverID:
                    GameState.getInstance().State = GameState.Mode.GAMEOVER;
                    return;
                default:
                    waittime = 0;
                    break;
            }

            GameState.getInstance().State = GameState.Mode.EVENT;
            evtrec = evttable.getEventRecord(id);
            if ((evtrec[EventTbl.jumpevtid] != "null") && (evtrec[EventTbl.jumpevtid] != ""))
            {
                EventExecRequest(evtrec[EventTbl.jumpevtid]);
            }

            execevtrec_comp = false;
            eventtype = evtrec[EventTbl.type];

            switch (eventtype)
            {
                case ExecEventTypes.WindowOpen:
                    msgstart = 0;
                    dispmsg = "";
                    EventImageWindow.getInstance().openWindow(evtrec[EventTbl.imgfile]);
                    break;
                case ExecEventTypes.CharPut:
                    Character charp = CharacterFactory.getInstance().GenerateChar(evtrec[EventTbl.chartype], screen, map);
                    charp.setMapPosition((int)evtrec[EventTbl.setx, TypeCode.Int32], (int)evtrec[EventTbl.sety, TypeCode.Int32]);
                    GameEventMng.getInstance().addCharacter(charp);
                    GameState.getInstance().State = GameState.Mode.MAIN;
                    break;
                case ExecEventTypes.FlagOperation:
                    string upflag = evtrec[EventTbl.upflags];
                    if (upflag != "")
                    {
                        FlagConditionTbl.getInstance().setFlags(upflag.Split("\n".ToCharArray()));
                    }

                    string downflag = evtrec[EventTbl.downflags];
                    if (downflag != "")
                    {
                        FlagConditionTbl.getInstance().resetFlags(downflag.Split("\n".ToCharArray()));
                    }
                    GameState.getInstance().State = GameState.Mode.MAIN;
                    break;
                default:
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "sCxg " + eventtype + " ͖`łB");
            }
        }

        public void ClearEventQueue()
        {
            eventqueue.Clear();
        }

        public override void Control()
        {
            if (GameState.getInstance().State == GameState.Mode.TITLE)
            {
                return;
            }

            EventExecRequest();

            switch (GameState.getInstance().State)
            {
                case GameState.Mode.STAGEEND:
                    Control_StgEndEvent();
                    return;
                case GameState.Mode.CLEAR:
                    Control_StgClrEvent();
                    return;
                case GameState.Mode.GAMEOVER:
                    Control_GameOverEvent();
                    return;
                case GameState.Mode.STGSTART:
                    Control_StgStartEvent();
                    return;
                case GameState.Mode.GAMECLEAR:
                    Control_GameClearEvent();
                    return;
                default:
                    break;
            }

            switch (eventtype)
            {
                case ExecEventTypes.WindowOpen:
                    Control_WinEvent();
                    break;
                case ExecEventTypes.CharPut:
                    break;
                default:
                    return;
            }
        }

        private void Control_WinEvent()
        {
            if (execevtrec_comp == true)
            {
                evtrec = evttable.getEventRecord();
                msgstart = 0;
                dispmsg = "";
                execevtrec_comp = false;
            }

            if (evtrec == null)
            {
                return;
            }

            if ((dispmsg == "") && (EventImageWindow.getInstance().IsOpenComp == true))
            {
                dispmsg = Util.DispStringBuild(evtrec[EventTbl.msgtext], fontwidth);
            }
        }

        public void Control_StgStartEvent()
        {
            key.Update();

            if ((key.IsPull(5) == true) && (FormInfo.getInstance().IsActive == true))
            {
                framecounter = 0;
                GameState.getInstance().State = GameState.Mode.MAIN;
                return;
            }
            else
            {
                framecounter++;
            }
        }

        public void Control_StgEndEvent()
        {
            GameState.getInstance().State = GameState.Mode.CLEAR;

            eventqueue.Clear();//Xe[W܂ăCxgێȂ̂ŁAL[NAB
        }

        public void Control_StgClrEvent()
        {
            key.Update();
            if ( (key.IsPull(5) == true) && (waittime <= 0) &&
                 (FormInfo.getInstance().IsActive == true) )
            {
                waittime = 0;
                framecounter = 0;
                GameState.getInstance().State = GameState.Mode.SHORTSTORY;
                GameEventMng.getInstance().UpdateNextStage();
                GameState.getInstance().stage = GameEventMng.getInstance().NextStage;
                return;
            }
            waittime--;
        }

        public void Control_GameOverEvent()
        {
            key.Update();

            if ( (key.IsPull(5) == true) && (waittime <= 0) &&
                 (FormInfo.getInstance().IsActive == true) )
            {
                waittime = 0;
                ClearEventQueue();
                GameEventMng.getInstance().clearHitcounters();
                GameState.getInstance().State = GameState.Mode.TITLE;
            }
            waittime--;
        }

        public void Control_GameClearEvent()
        {
            ClearEventQueue();
            GameEventMng.getInstance().clearHitcounters();
            GameState.getInstance().State = GameState.Mode.TITLE;
        }

        public override void DrawLayer()
        {
            if (GameState.getInstance().State == GameState.Mode.TITLE)
            {
                return;
            }

            switch (GameState.getInstance().State)
            {
                case GameState.Mode.STAGEEND:
                    DrawLayer_StgEndEvent();
                    return;
                case GameState.Mode.CLEAR:
                    DrawLayer_StgClrEvent();
                    return;
                case GameState.Mode.GAMEOVER:
                    DrawLayer_GameOverEvent();
                    return;
                case GameState.Mode.STGSTART:
                    DrawLayer_StgStartEvent();
                    return;
                case GameState.Mode.GAMECLEAR:
                    DrawLayer_GameClearEvent();
                    return;
                default:
                    break;
            }

            switch (eventtype)
            {
                case ExecEventTypes.WindowOpen:
                    DrawLayer_WinEvent();
                    break;
                case ExecEventTypes.CharPut:
                    break;
                default:
                    return;
            }
        }

        public void DrawLayer_WinEvent()
        {
            if (evtrec == null)
            {
                if (EventImageWindow.getInstance().IsOpenComp == true)
                {
                    EventImageWindow.getInstance().closeWindow();
                }
                else if (GameState.getInstance().State == GameState.Mode.EVENT)
                {
                    GameState.getInstance().State = GameState.Mode.MAIN;
                }

                return;
            }

            if (EventImageWindow.getInstance().IsOpenComp == false)
            {
                EventImageWindow.getInstance().openWindow();
                return;
            }

            EventImageWindow.getInstance().openWindow();

            screen.Blt(msgwintexture, 0, CornerCrashDefine.WINDOW_HEIGHT - CornerCrashDefine.MSG_WINDOW_HEIGHT);

            framecounter++;
            key.Update();
            if ( (key.IsPress(5) == true) && (framecounter > 4) &&
                 (FormInfo.getInstance().IsActive == true) )
            {
                msgstart = dispmsg.IndexOf("\n", msgstart);

                if ((msgstart >= 0) && (msgstart < dispmsg.Length))
                {
                   framecounter = 0;
                   msgstart++;
                }
                else
                {
                    execevtrec_comp = true;
                    return;
                }
            }

            int start = msgstart;

            for (int i = 0; i < CornerCrashDefine.MSG_WINDOW_LINECNT; i++)
            {
                int nline = 0;
                
                nline = dispmsg.IndexOf("\n", start);

                if ((start == nline) || ((dispmsg.Length > start) == false))
                {
                    break;
                }

                if (nline >= 0)
                {
                    msgtexture = fontrp.GetTexture(dispmsg.Substring(start, nline - start), 0);
                }
                else
                {
                    msgtexture = fontrp.GetTexture(dispmsg.Substring(start), 0);
                }

                screen.Blt(msgtexture, fontwidth,
                    (int)((i * fontwidth) * 1.5) + CornerCrashDefine.WINDOW_HEIGHT - CornerCrashDefine.MSG_WINDOW_HEIGHT);

                msgtexture.Release();

                if (nline < 0)
                {
                    break;
                }

                start = nline + 1;
            }
        }

        public void DrawLayer_StgStartEvent()
        {
            if (GameState.getInstance().State != GameState.Mode.STGSTART)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "ĂяȍԂsłB");
            }

            int stage = GameState.getInstance().stage;
            msgtexture = fontrp.GetTexture("Stage " + stage, 0);
            screen.Blt(msgtexture,
                (CornerCrashDefine.WINDOW_WIDTH - (stageclrmsg.Length * fontwidth))
                / 2, (CornerCrashDefine.WINDOW_HEIGHT / 2) - (fontwidth / 2));
            msgtexture.Release();

            msgtexture = smallfontrp.GetTexture(stagestartmsg, 0);
            screen.Blt(msgtexture,
                (CornerCrashDefine.WINDOW_WIDTH - (stagestartmsg.Length * CornerCrashDefine.SMALLFONT_WIDTH))
                / 2, (CornerCrashDefine.WINDOW_HEIGHT / 2) - (CornerCrashDefine.SMALLFONT_WIDTH / 2) +
                (int)(fontwidth * 1.5));
            msgtexture.Release();
        }

        public void DrawLayer_StgEndEvent()
        {
            //ȂB
        }

        public void DrawLayer_StgClrEvent()
        {
            if (GameState.getInstance().State != GameState.Mode.CLEAR)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "ĂяȍԂsłB");
            }
            msgtexture = bigfontrp.GetTexture(stageclrmsg, 0);
            screen.Blt(msgtexture,
                (CornerCrashDefine.WINDOW_WIDTH - (stageclrmsg.Length * CornerCrashDefine.BIGFONT_WIDTH))
                / 2, (CornerCrashDefine.WINDOW_HEIGHT / 2) - (CornerCrashDefine.BIGFONT_WIDTH / 2));
            msgtexture.Release();
        }

        public void DrawLayer_GameOverEvent()
        {
            string gameover = "Q[I[o[II";
            msgtexture = bigfontrp.GetTexture(gameover, 0);
            screen.Blt(msgtexture, 
                (CornerCrashDefine.WINDOW_WIDTH - (gameover.Length * CornerCrashDefine.BIGFONT_WIDTH)) / 2,
                (CornerCrashDefine.WINDOW_HEIGHT - CornerCrashDefine.BIGFONT_WIDTH) / 2);

            msgtexture.Release();
        }

        public void DrawLayer_GameClearEvent()
        {

        }

        public override LayerType Type
        {
            get
            {
                return LayerType.EVENTEXEC;
            }
        }
    }
}
