using System;
using System.Collections.Generic;
using System.Text;

namespace CornerCrash
{
    class GameEventMng
    {
        protected static GameEventMng instance;

        protected CharacterLayer charlayer;
        protected PlayerChar player;
        protected EventExecLayer eventlayer;

        protected List<DataRecord> evttable;
        protected EvtConditionTbl mastertbl;
        protected HitCounters hitcounters;
        protected int nextstage;
        protected int firststg;

        protected Random ExecRnd;

        public const string StagegEndID = "stageend";
        public const string GameOverID = "gameover";
        public const string UhoHitID = "uhohit";
        public const string PoliceHitID = "policehit";

        protected GameEventMng()
        {
            nextstage = 0;
        }

        public static GameEventMng getInstance()
        {
            if (instance == null)
            {
                throw new CornerCrashException(new GameEventMng(), System.Reflection.MethodBase.GetCurrentMethod(),
                    "CX^XĂ܂B");
            }
            return instance;
        }

        public static GameEventMng getInstance(EvtConditionTbl table)
        {

            if (instance != null)
            {
                return instance;
            }
            instance = new GameEventMng();
            instance.ExecRnd = new Random(System.Environment.TickCount);
            instance.mastertbl = table;
            return instance;
        }

        public void Init(int stage, PlayerChar playerchar, CharacterLayer chlayer, EventExecLayer evtlayer)
        {
            eventlayer = evtlayer;
            firststg = stage;
            Init(stage, playerchar, chlayer);
            hitcounters = new HitCounters();
            hitcounters.Init();
        }

        public void Init(int stage, PlayerChar playerchar, CharacterLayer chlayer)
        {
            instance.player = playerchar;
            
            instance.charlayer = chlayer;
            if (evttable != null)
            {
                evttable.Clear();
            }
            
            evttable = new List<DataRecord>();
            DataRecord rec;
            rec = mastertbl.getFirstRecord(stage);
            if (rec == null)
            {
                return;
            }
            evttable.Add(rec);
            while ((rec = mastertbl.getNextRecord(stage)) != null)
            {
                evttable.Add(rec);
            }
            evttable.Sort(delegate(DataRecord p1, DataRecord p2)
            {
                return ((int)p1[EvtConditionTbl.eventid, TypeCode.Int32] -
                    (int)p2[EvtConditionTbl.eventid, TypeCode.Int32]);
            });
            
        }

        public void InitMasterTbl(EvtConditionTbl table)
        {
            if (mastertbl != null)
            {
                mastertbl.Dispose();
            }

            mastertbl = table;
        }

        public void InitNextStage()
        {
            nextstage = firststg;
        }

        public void UpdateNextStage()
        {
            nextstage = GameState.getInstance().stage + 1;
        }

        public void addCharacter(Character character)
        {
            character.setCharacters(charlayer.Characters);
            charlayer.addCharacter(character);
        }

        public void execEvent()
        {
            if (GameState.getInstance().State != GameState.Mode.MAIN)
            {
                return;
            }

            if (LimitTimes.getInstance().CurTime == 0)
            {
                GameState.getInstance().State = GameState.Mode.GAMEOVER;
                return;
            }

            if (player.CurPosOfBGType == BGMap.BGType.GOAL)
            {
                eventlayer.EventExecRequest(StagegEndID);
                return;
            }

            for (int i = 0; i < evttable.Count; i++)
            {
                if ((int)(evttable[i][EvtConditionTbl.enable, TypeCode.Int32]) == 1)
                {
                    bool execflag = true;
                    foreach (string key in evttable[i].defkeys)
                    {
                        switch (key)
                        {
                            case EvtConditionTbl.eventid:
                                continue;
                            case EvtConditionTbl.stageid:
                                continue;
                            case EvtConditionTbl.enable:
                                continue;
                            case EvtConditionTbl.execevent:
                                continue;
                            case EvtConditionTbl.nextevtid :
                                continue;
                            case EvtConditionTbl.random :
                                if ((int)(evttable[i][key, TypeCode.Int32]) != 0)
                                {
                                    if ((int)(evttable[i][key, TypeCode.Int32]) > (ExecRnd.Next() % 10000))
                                    {
                                        //BiȂj
                                    }
                                    else
                                    {
                                        execflag = false;//sBstOfalseɁB
                                    }
                                }
                                break;
                            case EvtConditionTbl.upflagsid :
                                string upflag = evttable[i][EvtConditionTbl.upflagsid];
                                if (upflag == "")
                                {
                                    break;
                                }

                                if (FlagConditionTbl.getInstance().IsUpFlags(
                                    upflag.Split("\n".ToCharArray()))== true)
                                {
                                    //BiȂj
                                }
                                else
                                {
                                    execflag = false;//sBstOfalseɁB
                                }
                                break;
                            case EvtConditionTbl.downflagsid :
                                string downflag = evttable[i][EvtConditionTbl.downflagsid];
                                if (downflag == "")
                                {
                                    break;
                                }

                                if (FlagConditionTbl.getInstance().IsDownFlags(
                                    downflag.Split("\n".ToCharArray())) == true)
                                {
                                    //BiȂj
                                }
                                else
                                {
                                    execflag = false;//sBstOfalseɁB
                                }
                                break;
                            case EvtConditionTbl.time:
                                if ((int)(evttable[i][key, TypeCode.Int32]) != 0)
                                {
                                    if ((charlayer.Time / CornerCrashDefine.EVENTEXEC_TIME_BASE)
                                        > (int)(evttable[i][key, TypeCode.Int32]))
                                    {
                                        //BiȂj
                                    }
                                    else
                                    {
                                        execflag = false;//sBstOfalseɁB
                                    }
                                }
                                break;
                            case EvtConditionTbl.mapx:
                                if ( ((player.x / BGMap.MAPCHIP_WIDTH) == (int)evttable[i][key, TypeCode.Int32]) && 
                                     ((player.x / BGMap.MAPCHIP_WIDTH) == 
                                     ((player.x + player.sprite.width) / BGMap.MAPCHIP_WIDTH))
                                   )
                                {
                                    //BiȂj
                                }
                                else
                                {
                                    execflag = false;//sBstOfalseɁB
                                }
                                break;
                            case EvtConditionTbl.mapy:
                                if ( ((player.y / BGMap.MAPCHIP_HEIGHT) == (int)evttable[i][key, TypeCode.Int32]) &&
                                     ((player.y / BGMap.MAPCHIP_HEIGHT) == 
                                     ((player.y + player.sprite.height) / BGMap.MAPCHIP_HEIGHT))
                                   )
                                {
                                    //BiȂj
                                }
                                else
                                {
                                    execflag = false;//sBstOfalseɁB
                                }
                                break;
                            default:
                                if ((int)(evttable[i][key, TypeCode.Int32]) != 0)
                                {
                                    if (hitcounters.RefHitCounters(key) >= (int)(evttable[i][key, TypeCode.Int32]))
                                    {
                                        //BiȂj
                                    }
                                    else
                                    {
                                        execflag = false;//sBstOfalseɁB
                                    }
                                }
                                break;
                        }
                    }
                    if (execflag == true)
                    {
                        //CxgsB
                        if (evttable[i][EvtConditionTbl.execevent] == "0")
                        {
                            eventlayer.EventExecRequest(StagegEndID);
                        }
                        else
                        {
                            Console.WriteLine("Cxg : CxgID="
                                + (int)(evttable[i][EvtConditionTbl.eventid, TypeCode.Int32]));//
                            eventlayer.EventExecRequest(evttable[i][EvtConditionTbl.execevent]);
                        }
                        //ȉ͏ԑJځBsCxg𖳌ɐݒ肵AJڐ̃CxgLɂB
                        evttable[i][EvtConditionTbl.enable, TypeCode.Int32] = 0;
                        if ((int)(evttable[i][EvtConditionTbl.nextevtid, TypeCode.Int32]) == 0)
                        {
                            continue;
                        }

                        for (int j = 0; j < evttable.Count; j++)
                        {
                            int eventid = (int)evttable[j][EvtConditionTbl.eventid, TypeCode.Int32];
                            if (eventid == (int)(evttable[i][EvtConditionTbl.nextevtid, TypeCode.Int32]))
                            {
                                evttable[j][EvtConditionTbl.enable, TypeCode.Int32] = 1;
                            }
                        }
                    }
                }
            }
        }

        public void UhoHitEventExec()
        {
            eventlayer.EventExecRequest(UhoHitID);
        }

        public void PoliceHitEventExec()
        {
            eventlayer.EventExecRequest(PoliceHitID);
        }

        public void EventExecRequest(string id)
        {
            eventlayer.EventExecRequest(id);
        }

        public void charHitNotice(CornerCrashDefine.characterType uniqtype)
        {
            hitcounters.incHitcounter(uniqtype);
        }

        public void charHitNotice(CornerCrashDefine.characterType uniqtype, int count)
        {
            hitcounters.addHitcounter(uniqtype, count);
        }

        public int RefHitCounters(string key)
        {
            return hitcounters.RefHitCounters(key);
        }

        public int RefHitCounters(CornerCrashDefine.characterType key)
        {
            return hitcounters.RefHitCounters(key);
        }

        public int RefBackHitCounters(string key)
        {
            return hitcounters.RefBackHitCounters(key);
        }

        public int RefBackHitCounters(CornerCrashDefine.characterType key)
        {
            return hitcounters.RefBackHitCounters(key);
        }

        public void addBackHitcounter(CornerCrashDefine.characterType key, int cnt)
        {
            hitcounters.addBackHitcounter(key, cnt);
        }

        public void incBackHitcounter(CornerCrashDefine.characterType key)
        {
            hitcounters.incBackHitcounter(key);
        }

        public void clearHitcounter(CornerCrashDefine.characterType key)
        {
            hitcounters.clearHitcounter(key);
        }

        public void clearHitcounter(string key)
        {
            hitcounters.clearHitcounter(key);
        }

        public void clearHitcounters()
        {
            hitcounters.clearHitcounters();
        }

        public HitCounters HitCounters
        {
            get
            {
                return hitcounters;
            }
        }


        public int NextStage
        {
            get
            {
                return nextstage;
            }
        }
    }
}
