using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using YD = Yanesdk.Draw;

namespace CornerCrash
{
    class ImageLayers : IDisposable
    {
        private List<string> layers;
        private Dictionary<string, ImageLayerInfo> layerdic;
        private int layerno;
        private Bitmap image;
        private bool updatereq;

        public ImageLayers(Bitmap bmp)
        {
            layers = new List<string>();
            layerdic = new Dictionary<string, ImageLayerInfo>();
            layerno = 0;
            image = bmp;
            updatereq = false;
        }

        public ImageLayers(ImageLayers src)
        {
            ImageLayerInfo layer;

            for (int i = 0; i < src.layers.Count; i++)
            {
                layer = new ImageLayerInfo(src[src.layers[i]]);
                Add(src.layers[i], layer);
                layerno = src.layerno;
            }

            image = new Bitmap(src.image);

            if (layers.Count > 0)
            {
                updatereq = true;
            }
            else
            {
                updatereq = false;
            }
        }

        public bool IsSet(string key)
        {
            return layerdic.ContainsKey(key);
        }

        public void Add(string key, ImageLayerInfo val)
        {
            if (val == null)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "C[ւ̎QƂnullłB");
            }

            if (layerdic.ContainsKey(key) == true)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "̂̃C[ɓo^Ă܂B");
            }

            layerdic[key] = val;
            layers.Add(key);
            updatereq = true;
        }

        public void Remove(string key)
        {
            if (layerdic.ContainsKey(key) == false)
            {
                return;
            }
            
            layerdic[key].Dispose();
            layerdic.Remove(key);
            layers.Remove(key);

            updatereq = true;
        }

        public void setBmp(Bitmap bmp)
        {
            image = bmp;
            updatereq = true;
        }

        public void Save()
        {
            foreach (string key in layers)
            {
                layerdic[key].ReleaseImage();
            }
        }

        public void Resume()
        {
            foreach (string key in layers)
            {
                layerdic[key].ResumeImage();
            }

            updatereq = true;
        }

        public void Clear()
        {
            foreach (string key in layers)
            {
                layerdic[key].Dispose();
            }

            layerdic.Clear();
            layers.Clear();

            ClearBmp();
        }

        public void ClearBmp()
        {
            Graphics g = Graphics.FromImage(image);
            g.FillRectangle(Brushes.Black, 0, 0, image.Width, image.Height);

            g.Dispose();
            g = null;

            updatereq = true;
        }

        public void UpdateRequest()
        {
            updatereq = true;
        }

        public Bitmap getImage()
        {
            if (updatereq == true)
            {
                MakeImage();
            }

            return image;
        }

        public ImageLayerInfo this[string key]
        {
            get
            {
                if (layerdic.ContainsKey(key) == false)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "o^̃C[ " + key + " QƂ܂B");
                }

                return layerdic[key];
            }

            set
            {
                if (value == null)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "ݒ肵悤ƂC[ւ̎QƂnullłB");
                }
                
                if (layerdic.ContainsKey(key) == false)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "C[" + key + " ͖o^łB Add\bhœo^ĂB");
                }

                layerdic[key] = value;
                updatereq = true;
            }
        }

        private void setBegin()
        {
            layerno = 0;
        }

        private ImageLayerInfo getNext()
        {
            if (layers.Count <= layerno)
            {
                return null;
            }

            return layerdic[layers[layerno++]];
        }

        private void MakeImage()
        {
            ClearBmp();
            setBegin();

            ImageLayerInfo addimage;

            while ((addimage = getNext()) != null)
            {
                if (addimage.image != null)
                {
                    image = Util.LoadBmp(image, addimage.image, addimage.x, addimage.y);
                }
            }

            updatereq = false;
        }

        public virtual void Dispose()
        {
            foreach (string key in layers)
            {
                layerdic[key].Dispose();
            }

            layerdic.Clear();
            layers.Clear();
            image.Dispose();

            layerdic = null;
            layers = null;
            image = null;
        }
    }
}
