using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using YT = Yanesdk.Timer;
using YD = Yanesdk.Draw;
using Yanesdk.System;

namespace CornerCrash
{
    public partial class MainWindow : Form
    {
        YD.Win32Window window;
        YD.Screen2DGl screen;
        YD.FpsLayer fpsLayer;
        //eXgR[h
        //eXgR[hI
        MapDataFiles mapfiles;
        GameEventMng eventManager;
        List<BaseLayer> LayerList;
        PlayerChar player;
        BGMap map;
        CounterLayer CounterPrinter;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {
            FormInfo.getInstance().Set(this);
            FileArchiverRootZip.PassStatic = CornerCrashDefine.zipPass;

            if(Util.IsExist(CornerCrashDefine.ArchiveFile + ".zip") == true)
            {
                FileArchiverSingleton.getInstance().Archiver = new FileArchiverRootZip(
                        FileSys.ConcatPath(CornerCrashDefine.ArchiveFile,
                        FileSys.GetPureFileName(CornerCrashDefine.ArchiveFile)));
                FileSys.AddArchiver(FileArchiverSingleton.getInstance().Archiver);
            }

            this.ClientSize = new Size(CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT);
            
            window = new YD.Win32Window(MainScreen.Handle);

            screen = window.Screen;
            
            screen.SetClearColor(0, 0, 0);
            screen.BlendSrcAlpha();
            
            YT.FpsTimer fpsTimer = new YT.FpsTimer();
            fpsTimer.Fps = 30;
            fpsLayer = new YD.FpsLayer(fpsTimer);

            //eXgR[h
            //eXgR[hI
            EventImageWindow.getInstance().Screen = screen;
            EvtConditionTbl eventtbl = new EvtConditionTbl();
            eventtbl.ReadCsv("eventdata/eventdata.csv");
            eventManager = GameEventMng.getInstance(eventtbl);
            
            screen.Select();
            mapfiles = new MapDataFiles("mapdata/maps", "mapdata/exmap", 
                "mapdata/maplist.txt", "mapdata/chmaplist.txt", "mapdata/mvmaplist.txt");
            mapfiles.ReadMapFiles();

            if (LimitTimes.getInstance("conf/limittimes.txt").Length != mapfiles.MapCount)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "}bvt@CƑΉ鐧Ԑݒt@C̒`̐v܂B");
            }

            GameState.getInstance().stage = 1;
            int[][] charmap = mapfiles[GameState.getInstance().stage - 1].getCnvChMapData();
            int[][] mapdata = mapfiles[GameState.getInstance().stage - 1].getCnvMapData();
            int[][] mvdata = mapfiles[GameState.getInstance().stage - 1].getCnvMoveMapData();

            map = new BGMap(mapdata, mvdata);
            map.LoadBmp();
            player = new PlayerChar(screen, map);

            LayerList = new List<BaseLayer>();
            BGLayer BGLay = new BGLayer();
            BGLay.Init(map, player, screen);
            BGLay.baseChar = player;
            CharacterLayer CharLayer = new CharacterLayer(BGLay.Width, BGLay.Height);
            CharLayer.baseChar = player;
            player.setStartPosition(mapdata);
            CharLayer.addCharacter(player);
            CharLayer.setCharsToMap(screen, map, charmap);
            EventExecLayer eventlayer = new EventExecLayer(screen, map);
            eventlayer.Init("eventdata/execevent.csv");
            GamePointLayer PointPrinter = new GamePointLayer(screen, player);
            CounterPrinter = new CounterLayer(screen);
            LimitTimeLayer LimitTimePrinter = new LimitTimeLayer(screen);
            TitleLayer TitlePrinter = new TitleLayer(screen);
            screen.Unselect();
            player.setCharacters(CharLayer.Characters);
            //BlindLayer maskPrinter = new BlindLayer(screen, CharLayer.Characters);
            EffectLayer EffectPrinter = EffectLayer.getInstance();
            EffectPrinter.Init(screen);

            LayerList.Add(BGLay);
            LayerList.Add(CharLayer);
            //LayerList.Add(maskPrinter);
            LayerList.Add(eventlayer);
            LayerList.Add(PointPrinter);
            LayerList.Add(LimitTimePrinter);
            LayerList.Add(TitlePrinter);
            LayerList.Add(CounterPrinter);
            LayerList.Add(EffectPrinter);
            eventManager.Init(1, player, CharLayer, eventlayer);
            HitEvtDefTbl.GenInstance("eventdata/hitevent.csv");
            HitEvtDefTbl.getInstance().InitStage(GameState.getInstance().stage);
            GameState.getInstance().State = GameState.Mode.TITLE;
            CounterPrinter.RefCounterInit(GameState.getInstance().stage);

            FlagConditionTbl.GenInstance("conf/flagconf.csv");
            SinarioEngine.GenInstance(screen);

            eventManager.InitNextStage();
            GameState.getInstance().stage = 1;

            maintimer.Start();
        }

        private void maintimer_Tick(object sender, EventArgs e)
        {
            //C
            
            //FPS𖞂邩
            fpsLayer.FpsTimer.WaitFrame();
            if (fpsLayer.FpsTimer.ToBeSkip)
            {
                // w肵FPS𖞂Ȃ̂(d)`XLbv
                return;
            }

            //`揈
            screen.Select();
            screen.SetClearColor(0, 0, 0);
            screen.Clear();
            
            eventManager.execEvent();
            screen.BlendSrcAlpha();
            player.Control();

            if (GameState.getInstance().State == GameState.Mode.SHORTSTORY)
            {
                maintimer.Stop();
                FlagConditionTbl.getInstance().setFlags(GameState.getInstance().stage);

                SinarioEngine engine = SinarioEngine.getInstance();
                if (engine.Enable == false)
                {
                    GameState.getInstance().State = GameState.Mode.GAMECLEAR;
                    maintimer.Start();
                    return;
                }

                engine.Start();

                SSTimer.Start();
                return;
            }

            if ((GameState.getInstance().State == GameState.Mode.GAMEOVER) ||
                (GameState.getInstance().State == GameState.Mode.GAMECLEAR))
            {
                eventManager.InitNextStage();
                GameState.getInstance().stage = 1;
                CounterPrinter.RefCounterInit(GameState.getInstance().stage);
                SinarioEngine.getInstance().Init();
                SinarioEngine.getInstance().SectionInit();
                HitEvtDefTbl.getInstance().InitStage(GameState.getInstance().stage);
            }

            if (GameState.getInstance().State == GameState.Mode.NEXTSTAGE)
            {
                if ((eventManager.NextStage == 0) || (mapfiles.MapCount < eventManager.NextStage))
                {
                    //GfBOʂցH
                    //̏ƂẮASC[JăvOI
                    foreach (BaseLayer layer in LayerList)
                    {
                        layer.Dispose();
                    }
                    LayerList.Clear();
                    maintimer.Stop();
                    MessageBox.Show("ȏXe[W܂BEBhEăQ[I܂B");

                    this.Close();

                    return;
                }

                BGLayer BGLay = null;
                CharacterLayer CharLayer = null;
                int[][] mapdata = null;
                int[][] mvdata = null;
                for (int i = 0; i < LayerList.Count; i++)
                {
                    if(LayerList[i].Type == BaseLayer.LayerType.BG)
                    {
                        LayerList[i].Dispose();
                        LayerList.RemoveAt(i);
                        mapdata = mapfiles[eventManager.NextStage - 1].getCnvMapData();
                        mvdata = mapfiles[eventManager.NextStage - 1].getCnvMoveMapData();

                        map = new BGMap(mapdata, mvdata);
                        map.LoadBmp();
                        BGLay = new BGLayer();
                        BGLay.Init(map, player, screen);
                        BGLay.baseChar = player;
                        LayerList.Insert(i, BGLay);
                    }

                    if (LayerList[i].Type == BaseLayer.LayerType.CHARACTER)
                    {
                        if (BGLay == null)
                        {
                            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                                "wiC[OɃLN^[C[̐sƂ܂B");
                        }

                        LayerList[i].Dispose();
                        LayerList.RemoveAt(i);
                        int[][] charmap = mapfiles[eventManager.NextStage - 1].getCnvChMapData();
                        CharLayer = new CharacterLayer(BGLay.Width, BGLay.Height);
                        player.setStartPosition(mapdata);
                        player.Init(map);
                        CharLayer.baseChar = player;
                        CharLayer.addCharacter(player);
                        CharLayer.setCharsToMap(screen, map, charmap);
                        LayerList.Insert(i, CharLayer);
                    }

                    if (LayerList[i].Type == BaseLayer.LayerType.BLIND)
                    {
                        if (CharLayer == null)
                        {
                            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                                "ECɐݒ肷LN^[XgۗLLN^[COłB");
                        }
                        BlindLayer blindlay = (BlindLayer)LayerList[i];
                        blindlay.InitPos();
                        blindlay.Characters = CharLayer.Characters;
                    }

                }

                eventManager.Init(GameState.getInstance().stage, player, CharLayer);
                HitEvtDefTbl.getInstance().InitStage(GameState.getInstance().stage);
                GameEventMng.getInstance().clearHitcounters();

                System.GC.Collect(2);
                if (maintimer.Enabled == false)
                {
                    maintimer.Start();
                }
                GameState.getInstance().State = GameState.Mode.STGSTART;
                return;
            }

            foreach (BaseLayer layer in LayerList)
            {
                layer.Control();
                layer.DrawLayer();
            }
            screen.Update();
            Debug.LOGPRINT(this, System.Reflection.MethodBase.GetCurrentMethod(),
                " : " + GC.GetTotalMemory(false));
            System.GC.Collect();
        }

        private void SSTimer_Tick(object sender, EventArgs e)
        {
            SinarioEngine engine = SinarioEngine.getInstance();

            if (engine.Enable == false)
            {
                GameState.getInstance().State = GameState.Mode.NEXTSTAGE;
                CounterPrinter.RefCounterInit(GameState.getInstance().stage);

                System.GC.Collect(2);
                SSTimer.Stop();
                maintimer.Start();
                return;
            }

            if (engine.IsExit == false)
            {
                if (engine.ResumeReq == true)
                {
                    screen.Select();
                    engine.ResumeLayer();
                    screen.Unselect();
                }

                engine.Control();
            }
            else
            {
                GameState.getInstance().State = GameState.Mode.NEXTSTAGE;
                CounterPrinter.RefCounterInit(GameState.getInstance().stage);

                System.GC.Collect(2);
                SSTimer.Stop();
                maintimer.Start();
            }

            Debug.LOGPRINT(this, System.Reflection.MethodBase.GetCurrentMethod(),
                " : " + GC.GetTotalMemory(false));
            System.GC.Collect(2);
        }
    }
}