using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

using YD = Yanesdk.Draw;
using Yanesdk.System;

namespace CornerCrash
{
    class OnyanokoTexture : CharacterTexture
    {
        private const string icondir = "charimg/icon";
        protected static Dictionary<string, OnyanokoTexture> instance;

        public new enum TEXTURE
        {
            BOTTOM = 0,
            BOTTOM2,
            BOTTOM1,
            LEFT,
            LEFT2,
            LEFT1,
            TOP,
            TOP2,
            TOP1,
            RIGHT,
            RIGHT2,
            RIGHT1,
            HITICON,
            MAX,
        }

        public static OnyanokoTexture getInstance(string imgdir)
        {
            if (instance == null)
            {
                instance = new Dictionary<string, OnyanokoTexture>();
            }

            if (instance.ContainsKey(imgdir) == true)
            {
                return instance[imgdir];
            }

            instance[imgdir] = new OnyanokoTexture();
            instance[imgdir].Load(imgdir);

            return instance[imgdir];
        }

        public static bool IsChangeDirection(TEXTURE now, int past, string imgdir)
        {
            return instance[imgdir].IsChangeDirection((int)now, past);
        }

        protected void Load(string imgdir)
        {
            textureList = new Yanesdk.Draw.GlTexture[(int)TEXTURE.MAX];
            textureList[(int)TEXTURE.TOP] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.TOP1] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.TOP2] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.BOTTOM] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.BOTTOM1] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.BOTTOM2] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.LEFT] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.LEFT1] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.LEFT2] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.RIGHT] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.RIGHT1] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.RIGHT2] = new Yanesdk.Draw.GlTexture();
            textureList[(int)TEXTURE.HITICON] = new Yanesdk.Draw.GlTexture();

            if (Util.IsExist(imgdir + "/" + "fullset.gif") == true)
            {
                LoadFromBMP(imgdir);
            }
            else
            {
                LoadFromFile(imgdir);
            }
            textureList[(int)TEXTURE.HITICON].Load(icondir + "/" + "hit.gif");
        }

        protected void LoadFromFile(string imgdir)
        {

            textureList[(int)TEXTURE.TOP].Load(imgdir + "/" + "top.gif");
            textureList[(int)TEXTURE.TOP1].Load(imgdir + "/" + "top1.gif");
            textureList[(int)TEXTURE.TOP2].Load(imgdir + "/" + "top2.gif");
            textureList[(int)TEXTURE.BOTTOM].Load(imgdir + "/" + "bottom.gif");
            textureList[(int)TEXTURE.BOTTOM1].Load(imgdir + "/" + "bottom1.gif");
            textureList[(int)TEXTURE.BOTTOM2].Load(imgdir + "/" + "bottom2.gif");
            textureList[(int)TEXTURE.LEFT].Load(imgdir + "/" + "left.gif");
            textureList[(int)TEXTURE.LEFT1].Load(imgdir + "/" + "left1.gif");
            textureList[(int)TEXTURE.LEFT2].Load(imgdir + "/" + "left2.gif");
            textureList[(int)TEXTURE.RIGHT].Load(imgdir + "/" + "right.gif");
            textureList[(int)TEXTURE.RIGHT1].Load(imgdir + "/" + "right1.gif");
            textureList[(int)TEXTURE.RIGHT2].Load(imgdir + "/" + "right2.gif");
        }

        protected void LoadFromBMP(string imgdir)
        {
            Bitmap TextureBMP = Util.LoadBmp(imgdir + "/" + "fullset.gif");
            int width = TextureBMP.Width / 3;
            int height = TextureBMP.Height / 4;

            Bitmap Bmpbuff = null;
            Graphics g = null;
            Rectangle dstrect = new Rectangle(0, 0, width, height);

            for (int y=0; y < 4; y++)
            {
                for (int x = 0; x < 3; x++)
                {
                    Bmpbuff = new Bitmap(width, height);
                    g = Graphics.FromImage(Bmpbuff);

                    g.DrawImage(TextureBMP, dstrect, x * width, y * height, width, height,
                        GraphicsUnit.Pixel);
                    textureList[(y * 3) + x].SetBitmap(Bmpbuff);
                    Bmpbuff.Dispose();
                    g.Dispose();
                }
            }
            System.GC.Collect(2);
        }
    }
}
