using System;
using System.Collections.Generic;
using System.Text;

using YD = Yanesdk.Draw;
using YI = Yanesdk.Input;

namespace CornerCrash
{
    class PlayerChar : Character
    {
        private long gamepoint;
        private int stamina;
        private int recoverywait;

        private const int recoverytime = 150;

        private const int staminaMax = 1800;

        enum StaminaConsumption
        {
            DEFAULT = 1,
            DASH = 3,
        }

        enum StaminaRecovery
        {
            DEFAULT = 180,
        }

        YI.Key1 key;
        YD.GlTexture[] textureList;

        public PlayerChar(YD.Screen2DGl pscreen, BGMap pMap)
            : base(pscreen, pMap)
        {
            mx = 6;
            my = 6;
            gamepoint = 1000;
            sprite.enable = true;
            key = new YI.Key1();
            key.AddKey(7, 0, (int)YI.KeyCode.z);
            stamina = 180;
            recoverywait = 0;
        }

        public void Init(BGMap pMap)
        {
            bgmap = pMap;
            stamina = staminaMax;
        }

        public override void LoadSprite()
        {
            textureList = PlayerTexture.getInstance().TextureList;
            sprite = new BaseSprite(screen, textureList[(int)PlayerTexture.TEXTURE.BOTTOM]);
            beforeDirection = (int)PlayerTexture.TEXTURE.BOTTOM;
        }

        public override void Control()
        {
            if (GameState.getInstance().State != GameState.Mode.MAIN)
            {
                return;
            }

            key.Update();
            recoverywait++;
            if (recoverywait >= recoverytime)
            {
                stamina += (int)StaminaRecovery.DEFAULT;
                if (stamina > staminaMax)
                {
                    stamina = staminaMax;
                }

                recoverywait = 0;
            }

            int decstamina = 0;

            if (FormInfo.getInstance().IsActive == false)
            {
                return;
            }

            if (key.IsPress(7))
            {
                mx = 8;
                my = 8;
                animintervalmax = animintervalbase / 2;
                decstamina = (int)StaminaConsumption.DASH;
            }
            else
            {
                mx = 4;
                my = 4;
                animintervalmax = animintervalbase;
                decstamina = (int)StaminaConsumption.DEFAULT;
            }

            if (stamina <= 0)
            {
                return;
            }

            if (key.IsPress(1))
            {
                decAnimInterval();
                MoveToTop(decstamina);
            }

            if (key.IsPress(2))
            {
                decAnimInterval();
                MoveToBottom(decstamina);
            }

            if (key.IsPress(3))
            {
                decAnimInterval();
                MoveToLeft(decstamina);
            }

            if (key.IsPress(4))
            {
                decAnimInterval();
                MoveToRight(decstamina);
            }
        }

        private bool IsChangeDirection(PlayerTexture.TEXTURE now)
        {
            return PlayerTexture.IsChangeDirection(now, beforeDirection);
        }

        private void MoveToTop(int decstamina)
        {
            if ((animinterval <= 0) || (IsChangeDirection(PlayerTexture.TEXTURE.TOP) == true))
            {
                if (backmvstate == BackMoveSt.MoveR)
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.TOP2]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.TOP2;
                    backmvstate = BackMoveSt.MoveL;
                }
                else
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.TOP1]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.TOP1;
                    backmvstate = BackMoveSt.MoveR;
                }
                animinterval = animintervalmax;
            }

            nowdirect = NowDirection.TOP;
            if (IsYMove(false) == false)
            {
                return;
            }
            stamina -= decstamina;
            y -= my;
        }

        private void MoveToBottom(int decstamina)
        {
            if ((animinterval <= 0) || (IsChangeDirection(PlayerTexture.TEXTURE.BOTTOM) == true))
            {
                if (backmvstate == BackMoveSt.MoveR)
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.BOTTOM2]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.BOTTOM2;
                    backmvstate = BackMoveSt.MoveL;
                }
                else
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.BOTTOM1]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.BOTTOM1;
                    backmvstate = BackMoveSt.MoveR;
                }
                animinterval = animintervalmax;
            }

            nowdirect = NowDirection.BOTTOM;
            if (IsYMove(true) == false)
            {
                return;
            }
            stamina -= decstamina;
            y += my;
        }

        private void MoveToLeft(int decstamina)
        {
            if ((animinterval <= 0) || (IsChangeDirection(PlayerTexture.TEXTURE.LEFT) == true))
            {
                if (backmvstate == BackMoveSt.MoveR)
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.LEFT2]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.LEFT2;
                    backmvstate = BackMoveSt.MoveL;
                }
                else
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.LEFT1]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.LEFT1;
                    backmvstate = BackMoveSt.MoveR;
                }
                animinterval = animintervalmax;
            }

            nowdirect = NowDirection.LEFT;
            if (IsXMove(false) == false)
            {
                return;
            }
            stamina -= decstamina;
            x -= mx;
        }

        private void MoveToRight(int decstamina)
        {
            if ((animinterval <= 0) || (IsChangeDirection(PlayerTexture.TEXTURE.RIGHT) == true))
            {
                if (backmvstate == BackMoveSt.MoveR)
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.RIGHT2]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.RIGHT2;
                    backmvstate = BackMoveSt.MoveL;
                }
                else
                {
                    sprite.setTexture(textureList[(int)PlayerTexture.TEXTURE.RIGHT1]);
                    beforeDirection = (int)PlayerTexture.TEXTURE.RIGHT1;
                    backmvstate = BackMoveSt.MoveR;
                }
                animinterval = animintervalmax;
            }

            nowdirect = NowDirection.RIGHT;
            if (IsXMove(true) == false)
            {
                return;
            }
            x += mx;
        }

        public override void addGamePoint(int point)
        {
            gamepoint += point;
        }

        public override void negGamePoint(int point)
        {
            gamepoint -= point;
        }

        public long GamePoint
        {
            get
            {
                return gamepoint;
            }
        }

        public void setStartPosition(int[][] pMap)
        {
            for (int i = 0; i < pMap.Length; i++)
            {
                for (int j = 0; j < pMap[0].Length; j++)
                {
                    if (pMap[i][j] == 0x01)
                    {
                        x = BGMap.MAPCHIP_WIDTH * j;
                        y = BGMap.MAPCHIP_HEIGHT * i;
                        return;
                    }
                }
            }
        }

        protected override bool IsYMove(bool isadd)
        {
            if (stamina <= 0)
            {
                return false;
            }

            if (base.IsYMove(isadd) == false)
            {
                return false;
            }

            return true;
        }

        protected override bool IsXMove(bool isadd)
        {
            if (stamina <= 0)
            {
                return false;
            }

            if (base.IsXMove(isadd) == false)
            {
                return false;
            }

            return true;
        }

        public override bool chkIsinBoxAreaNextPos(int chkx, int chky)
        {
            for (int i = 0; i < characters.Count; i++)
            {
                if (characters[i] == this)
                {
                    continue;
                }

                if (characters[i].CharType != CornerCrashDefine.characterType.Kaisyain)
                {
                    continue;
                }

                if (characters[i].chkIsinBoxArea(chkx, chky, sprite.width, sprite.height) == true)
                {
                    return true;
                }
            }
            return false;
        }

        public override CornerCrashDefine.characterType CharType
        {
            get
            {
                return CornerCrashDefine.characterType.Player;
            }
        }
    }
}
