using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Yanesdk.System;
using YD = Yanesdk.Draw;
using YI = Yanesdk.Input;
using YT = Yanesdk.Timer;
using YS = Yanesdk.Sound;

namespace CornerCrash
{
    class SinarioEngine
    {
        public enum Page
        {
            Back = 0,
            Fore ,
        }

        private YD.FpsLayer fpsLayer;
        
        private YD.FontRepository fontrp;
        private YD.FontLoader fontldr;
        private YI.Key1 key;
        private YI.MouseInput mouse;
        private YD.Screen2DGl screen;
        
        private YD.GlTexture forepage;
        
        private YD.GlTexture backpage;

        private ImageLayers forelayers;
        private ImageLayers backlayers;

        private YT.GameTimer transTimer;
        private int transtime;
        private bool istranswait;
        private bool issoundwait;

        private YD.Textures msgtexture;
        private YD.GlTexture msgwintexture;

        private YS.Sound soundplayer;
        private YS.Sound bgmplayer;
        private string bgmname;

        private const string dirname = "sinario";
        private const int waitmax = 6;
        private int waittime;

        private static SinarioEngine instance;

        private int curtxtpnt;
        private int fontcnt;
        private int linecnt;

        private CurrentSinario curSinario;
        private Stack<CurrentSinario> sinariostack;
        private bool fetch;
        private bool exit;
        private bool isbreak;
        private bool enable;
        private bool resumeReq;

        private StringBuilder disptxt;
        private int fontwidth;

        public static char spacech;
        public static char tabch;
        public static char stagch;
        public static char etagch;
        public static char dquote;
        public static char escape;

        private SinarioEngine(YD.Screen2DGl pscreen)
        {
            curSinario = new CurrentSinario();
            sinariostack = new Stack<CurrentSinario>();
            screen = pscreen;

            msgtexture = new Yanesdk.Draw.Textures();
            msgwintexture = new Yanesdk.Draw.GlTexture();

            soundplayer = new Yanesdk.Sound.Sound();
            soundplayer.Volume = 0.3f;
            bgmplayer = new Yanesdk.Sound.Sound();
            bgmplayer.Volume = 0.3f;

            forepage = new Yanesdk.Draw.GlTexture();
            backpage = new Yanesdk.Draw.GlTexture();

            Graphics g;

            Bitmap forebmp = new Bitmap(
                CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT);
            g = Graphics.FromImage(forebmp);
            g.DrawRectangle(new Pen(Color.Black), 0, 0, forebmp.Width, forebmp.Height);

            forepage.SetBitmap(forebmp);
            g.Dispose();
            g = null;

            Bitmap backbmp = new Bitmap(
                CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT);
                
            g = Graphics.FromImage(backbmp);
            g.DrawRectangle(new Pen(Color.Black), 0, 0, backbmp.Width, backbmp.Height);

            g.Dispose();
            g = null;

            forelayers = new ImageLayers(forebmp);
            backlayers = new ImageLayers(backbmp);

            YT.FpsTimer fpsTimer = new YT.FpsTimer();
            fpsTimer.Fps = 30;
            fpsLayer = new YD.FpsLayer(fpsTimer);

            transTimer = new Yanesdk.Timer.GameTimer();
            transTimer.Time = 0;
            transtime = 0;
            istranswait = false;

            issoundwait = false;

            fetch = false;
            exit = false;
            isbreak = false;
            resumeReq = false;

            fontwidth = 24;
            YD.SmartFontLoader smfldr = new Yanesdk.Draw.SmartFontLoader();
            fontldr = smfldr.LoadDefFile("mem:msgothic.ttc, " + fontwidth + ", 0, 0");
            fontrp = new YD.FontRepository(delegate { return new YD.GlTexture(); });
            fontrp.SetLoader(fontldr, 0);

            key = new Yanesdk.Input.Key1();
            if (Yanesdk.System.Platform.IsLinux == true)
            {
                mouse = new Yanesdk.Input.MouseInput();
            }
            else
            {
                mouse = new Yanesdk.Input.MouseInput(FormInfo.getInstance().Get());
            }

            spacech = Char.Parse(" ");
            tabch = Char.Parse("\t");
            stagch = Char.Parse("[");
            etagch = Char.Parse("]");
            dquote = Char.Parse("\"");
            escape = Char.Parse("\\");

            Init();
            SectionInit();
        }

        public void Init()
        {
            curSinario.file = "start.txt";
            curSinario.section = "";
            disptxt = new StringBuilder("");
            sinariostack.Clear();
            curtxtpnt = 0;
            fontcnt = 0;
            linecnt = 0;
            waittime = waitmax;
            fetch = false;
            exit = false;
            isbreak = false;
            enable = true;
            istranswait = false;
            issoundwait = false;
            resumeReq = false;
            LayerInit();
        }

        public void Exit()
        {
            exit = true;
            if (backpage.Loaded == true)
            {
                backpage.Release();
            }

            if ( (soundplayer.Loaded == true) ||
                 (bgmplayer.Loaded == true) )
            {
                YS.Sound.StopAll();
            }

            if (soundplayer.Loaded == true)
            {
                soundplayer.Release();
            }

            if (bgmplayer.Loaded == true)
            {
                bgmplayer.Release();
            }

            transTimer.Pause();
            transTimer.Time = 0;
            istranswait = false;
            issoundwait = false;
        }

        public void Break()
        {
            fetch = false;
            isbreak = true;
        }

        public void TransWait()
        {
            if (backpage.Loaded == false)
            {
                return;
            }

            Break();
            istranswait = true;
        }

        public void SoundWait()
        {
            if (soundplayer.IsPlaying() == false)
            {
                return;
            }

            Break();
            issoundwait = true;
        }

        public void SetLF()
        {
            disptxt.Append("\n");
            linecnt++;
            fontcnt = 0;
        }

        public void InBGM(string filename, int time)
        {
            bgmname = filename;
            filename = FileSys.ConcatPath("bgm/", filename);
            bgmplayer.Load(filename, -1);
            bgmplayer.PlayFade(time);
        }

        public void OutBGM(int time)
        {
            YS.Sound.FadeMusic(time);
            bgmname = null;
        }

        public void PlayBGM(string filename)
        {
            bgmname = filename;
            filename = FileSys.ConcatPath("bgm/", filename);
            bgmplayer.Load(filename, -1);
            bgmplayer.Play();
        }

        public void StopBGM()
        {
            YS.Sound.StopMusic();
            bgmname = null;
        }

        public void PlaySound(string filename)
        {
            filename = FileSys.ConcatPath("sound/", filename);
            soundplayer.Load(filename);
            soundplayer.Play();
        }

        public void StopSound()
        {
            soundplayer.Stop();
        }

        public void LayerInit()
        {
            forepage.Release();
            forelayers.Clear();
            backlayers.Clear();
            resumeReq = false;
        }

        public void SaveLayer()
        {
            forelayers.Save();
            backlayers.Save();
            resumeReq = true;
        }

        public void ResumeLayer()
        {
            Graphics g;

            Bitmap forebmp = new Bitmap(
                CornerCrashDefine.WINDOW_WIDTH, CornerCrashDefine.WINDOW_HEIGHT);
            g = Graphics.FromImage(forebmp);
            g.DrawRectangle(new Pen(Color.Black), 0, 0, forebmp.Width, forebmp.Height);

            forelayers.setBmp(forebmp);
            g.Dispose();
            g = null;

            forelayers.Resume();
            forepage.Release();
            forepage.SetBitmap(forelayers.getImage());
            backlayers.Resume();

            if (bgmname != null)
            {
                InBGM(bgmname, 1000);
            }

            resumeReq = false;
        }

        public void LoadLayer(string key, string imagepath, int x, int y, Page page)
        {
            imagepath = FileSys.ConcatPath("sinario/image/", imagepath);

            if (page == Page.Fore)
            {
                if (forelayers.IsSet(key) == true)
                {
                    forelayers[key].x = x;
                    forelayers[key].y = y;
                    forelayers[key].LoadImage(imagepath);
                }
                else
                {
                    forelayers.Add(key, new ImageLayerInfo(imagepath, x, y));
                }

                forepage.Release();
                forepage.SetBitmap(forelayers.getImage());
            }
            else
            {
                if (backlayers.IsSet(key) == true)
                {
                    backlayers[key].x = x;
                    backlayers[key].y = y;
                    backlayers[key].LoadImage(imagepath);
                }
                else
                {
                    backlayers.Add(key, new ImageLayerInfo(imagepath, x, y));
                }

                backlayers.UpdateRequest();
            }
        }

        public void ClearLayers(Page page)
        {
            if (page == Page.Fore)
            {
                forelayers.Clear();
                forepage.Release();
                forepage.SetBitmap(forelayers.getImage());
            }
            else
            {
                backlayers.Clear();
            }
        }

        public void ClearLayer(string key, Page page)
        {
            if (page == Page.Fore)
            {
                if (forelayers.IsSet(key) == true)
                {
                    forelayers.Remove(key);
                    forepage.Release();
                    forepage.SetBitmap(forelayers.getImage());
                }
            }
            else
            {
                if (backlayers.IsSet(key) == true)
                {
                    backlayers.Remove(key);
                }
            }
        }

        public void CopyLayer(Page dst)
        {
            if (dst == Page.Fore)
            {
                forelayers = new ImageLayers(backlayers);
                forepage.Release();
                forepage.SetBitmap(forelayers.getImage());
            }
            else
            {
                backlayers = new ImageLayers(forelayers);
            }
        }

        public void FadeinRequest(int time)
        {
            if (backpage.Loaded == true)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "tF[hCortF[hAEgsɁAVȃtF[hCvs܂B");
            }

            backpage.SetBitmap(backlayers.getImage());

            transtime = time;
            transTimer.Reset();
        }

        public void FadeoutRequest(int time)
        {
            if (backpage.Loaded == true)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "tF[hCortF[hAEgsɁAVȃtF[hAEgvs܂B");
            }

            backlayers.Clear();
            backpage.SetBitmap(backlayers.getImage());
            
            transtime = time;
            transTimer.Reset();
        }

        private int getBlendAlpha()
        {
            if (backpage.Loaded == false)
            {
                return 255;
            }

            int alpha = 0;

            if (transTimer.Time > 0)
            {
                alpha = (transTimer.Time * 255) / transtime;
            }

            if (alpha > 255)
            {
                alpha = 255;
            }

            return alpha;
        }

        private void DrawLayer()
        {
            screen.Blt(forepage);

            if (backpage.Loaded == false)
            {
                return;
            }

            int alpha = getBlendAlpha();

            TransTexture(alpha);

            if (alpha == 255)
            {
                backpage.Release();
                FlipLayer();
                transTimer.Pause();
                transTimer.Time = 0;
                istranswait = false;
            }
        }

        private void TransTexture(int alpha)
        {
            int r, g, b, a;

            screen.GetColor(out r, out g, out b, out a);
            screen.SetColor(r, g, b, alpha);

            screen.Blt(backpage);

            screen.SetColor(r, g, b, a);
        }

        private void FlipLayer()
        {
            ImageLayers tmplayers = forelayers;
            forelayers = backlayers;
            backlayers = tmplayers;

            forepage.Release();
            forepage.SetBitmap(forelayers.getImage());
            backlayers.UpdateRequest();
        }

        public static void GenInstance(YD.Screen2DGl pscreen)
        {
            if (instance != null)
            {
                throw new CornerCrashException(instance, System.Reflection.MethodBase.GetCurrentMethod(),
                    "CX^X͂łɐς݂łB");
            }

            instance = new SinarioEngine(pscreen);
        }

        public static SinarioEngine getInstance()
        {
            if (instance == null)
            {
                throw new CornerCrashException("SinarioEnginẽCX^XĂ܂BB");
            }

            return instance;
        }

        private string getSinario()
        {
            string path = FileSys.ConcatPath(dirname, curSinario.file);
            if (Util.IsExist(path) == false)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "ViIt@C " + path + "܂B");
            }

            byte[] buff = null;

            if (Util.IsExist(path) == true)
            {
                buff = FileSys.Read(path);
            }

            return Util.CnvByteArrayToUTF8Str(buff);
        }

        private int ParseTag(int index, out string name, out Dictionary<string, string> elm)
        {
             string tagname = "";
            int nprm = index;
            Dictionary<string, string> tagelm = new Dictionary<string, string>();

            int chpnt = index;
            while (chpnt < curSinario.sinario.Length)
            {
                if ((curSinario.sinario[chpnt] == spacech) ||
                    (curSinario.sinario[chpnt] == tabch) ||
                    (curSinario.sinario[chpnt] == etagch))
                {
                    break;
                }
                chpnt++;
            }
                
            if (chpnt == index)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "^OłB");
            }

            if ( (chpnt >= curSinario.sinario.Length) &&
                 (curSinario.sinario[chpnt] != etagch) )
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "^OĂ܂B");
            }

            tagname = curSinario.sinario.Substring(index, chpnt - index);
            index = chpnt;
            //fobOo start
            Debug.LOGPRINT(this, System.Reflection.MethodBase.GetCurrentMethod(),
                "^O : " + tagname);
            //fobOo end

            string elmname = "";
            string elmprm = "";
            int i = 0;
            for (i = index; i < curSinario.sinario.Length; i++)
            {
                if (curSinario.sinario[i] == etagch)
                {
                    break;
                }

                if ((curSinario.sinario[i] == spacech) ||
                   (curSinario.sinario[i] == tabch))
                {
                    continue;
                }

                if ((chpnt = curSinario.sinario.IndexOf("=", i)) == -1)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "^O" + tagname + "̃p[^̋؂L܂B");
                }

                elmname = curSinario.sinario.Substring(i, chpnt - i);
                i = chpnt + 1;
                elmprm = ParseTagPrm(i, out nprm);
                tagelm[elmname] = elmprm;
                i = nprm;

                //fobOo start
                Debug.LOGPRINT(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    " : " + elmname + "=" + elmprm);
                //fobOo end
                if (curSinario.sinario[i] == etagch)
                {
                    break;
                }
            }
            name = tagname;
            elm = tagelm;
            return i ;
        }

        private string ParseTagPrm(int index, out int newindex)
        {
            StringBuilder retstr = new StringBuilder("");
            int cur = 0;

            newindex = index;
            cur = index;
            if (curSinario.sinario[index] == dquote)
            {
                cur++;
                index++;

                for (int i = index; i < curSinario.sinario.Length; i++)
                {
                    if (curSinario.sinario[i] == escape)
                    {
                        if (curSinario.sinario[i + 1] == dquote)
                        {
                            retstr.Append(curSinario.sinario.Substring(cur, i - cur));
                            retstr.Append(dquote);
                            i++;
                            cur = i + 1;
                        }
                        else if (curSinario.sinario[i + 1] == escape)
                        {
                            retstr.Append(curSinario.sinario.Substring(cur, i - cur));
                            retstr.Append(escape);
                            i++;
                            cur = i;
                        }
                        else
                        {
                            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                                "GXP[vɑ" + curSinario.sinario[i + 1] + "͕słB");
                        }
                    }
                    else if (curSinario.sinario[i] == dquote)
                    {
                        retstr.Append(curSinario.sinario.Substring(cur, i - cur));
                        newindex = i;
                        return retstr.ToString();
                    }
                }
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "_uNH[gĂ܂B");
            }
            else
            {
                cur = index;
                while (cur < curSinario.sinario.Length)
                {
                    if ((curSinario.sinario[cur] == spacech) ||
                        (curSinario.sinario[cur] == tabch) ||
                        (curSinario.sinario[cur] == etagch))
                    {
                        break;
                    }
                    cur++;
                }

                if ((cur >= curSinario.sinario.Length) &&
                     (curSinario.sinario[cur] != etagch))
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "^OĂ܂B");
                }

                newindex = cur;
                return curSinario.sinario.Substring(index, cur - index);
            }            
        }

        public int findEndSection(int i)
        {
            int level = 1;
            int cur = i;
            string tagname;
            Dictionary<string, string> tagelm;
            
            for (; i < curSinario.sinario.Length; i++)
            {
                if ((cur = curSinario.sinario.IndexOf(stagch, i)) == -1)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "ZNV̏I肪܂łB");
                }

                i = cur;
                if ((curSinario.sinario.Substring(i + 1,
                        SinarioTag.TagName.section.Length + 1) == SinarioTag.TagName.section + " ") ||
                    (curSinario.sinario.Substring(i + 1,
                        SinarioTag.TagName.section.Length + 1) == SinarioTag.TagName.section + "\t"))
                {
                    ParseTag(i, out tagname, out tagelm);

                    if (tagelm.ContainsKey("name") == false)
                    {
                        throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                            "ZNVȂ^O܂B");
                    }
                    level++;
                }
                else if(curSinario.sinario.Substring(i + 1,
                    SinarioTag.TagName.endsection.Length) == SinarioTag.TagName.endsection)
                {
                    level--;
                }

                i = SinarioTag.findTagEnd(i + 1, curSinario.sinario);

                if (level == 0)
                {
                    return i;
                }
            }
            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                "ZNV̏I肪܂łB");
        }

        private int findSection(string section)
        {
            if (section == "")
            {
                return 0;
            }

            if (section[0] == stagch)
            {
                throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                    "u[vŎn܂ZNV͎gpł܂B");
            }

            int cur = 0;
            string tagname;
            Dictionary<string, string> tagelm;

            for (int i = 0; i < curSinario.sinario.Length; i++)
            {
                if ((cur = curSinario.sinario.IndexOf(stagch, i)) == -1)
                {
                    throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                        "ZNVȂ^O܂B");
                }

                i = cur;

                if ( (curSinario.sinario.Substring(i + 1,
                    SinarioTag.TagName.section.Length + 1) == SinarioTag.TagName.section + " ") ||
                     (curSinario.sinario.Substring(i + 1,
                    SinarioTag.TagName.section.Length + 1) == SinarioTag.TagName.section + "\t"))
                    
                {
                    ParseTag(i, out tagname, out tagelm);

                    if (tagelm.ContainsKey("name") == false)
                    {
                        throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                            "ZNVȂ^O܂B");
                    }

                    if (tagelm["name"] == section)
                    {
                        curSinario.pnt = SinarioEngine.getInstance().findEndSection(i + 1);
                        sinariostack.Push(curSinario);
                        curSinario.section = section;
                        return i;
                    }
                    else
                    {
                        i = SinarioTag.findTagEnd(i, curSinario.sinario);
                    }
                }
                else
                {
                    i = SinarioTag.findTagEnd(i, curSinario.sinario);
                }
            }
            throw new CornerCrashException(this, System.Reflection.MethodBase.GetCurrentMethod(),
                "ZNV : " + section + "܂łB");
        }

        public int SectionInit()
        {
            curSinario.sinario = getSinario();
            curtxtpnt = findSection(curSinario.section);

            return curtxtpnt;
        }

        public void Start()
        {
            disptxt = new StringBuilder("");
            fontcnt = 0;
            linecnt = 0;
            exit = false;
            isbreak = false;
            fetch = false;
            waittime = waitmax;
            istranswait = false;
            issoundwait = false;
            Exec();
            DrawLayer();
        }
        
        public void Control()
        {
            screen.Select();

            if (msgwintexture.Loaded == false)
            {
                msgwintexture.Load("windowimg/messagewindow.png");
            }

            if (soundplayer.IsPlaying() == false)
            {
                soundplayer.Release();
                issoundwait = false;
            }

            if (bgmplayer.IsPlaying() == false)
            {
                bgmplayer.Release();
            }

            key.Update();
            mouse.Update();

            if(FormInfo.getInstance().IsActive == true)
            {
                if ( ( (key.IsPress(5) == true) || (mouse.IsLeave(Yanesdk.Input.MouseInput.Button.Left) == true) ) && 
                     (istranswait == false) && (issoundwait == false))
                {
                    if (linecnt >= CornerCrashDefine.MSG_WINDOW_LINECNT)
                    {
                        disptxt.Remove(0, disptxt.ToString().IndexOf("\n") + 1);
                        linecnt--;
                    }
                    fetch = true;
                    waittime = waitmax;
                }
            }

            if (fetch == true)
            {
                Exec();
            }

            //FPS𖞂邩
            fpsLayer.FpsTimer.WaitFrame();
            if (fpsLayer.FpsTimer.ToBeSkip)
            {
                screen.Unselect();
                // w肵FPS𖞂Ȃ̂(d)`XLbv
                return;
            }

            screen.SetClearColor(0, 0, 0);
            screen.Clear();

            screen.BlendSrcAlpha();

            DrawLayer();

            if (waittime > 0)
            {
                waittime--;
            }

            screen.Blt(msgwintexture, 0, CornerCrashDefine.WINDOW_HEIGHT - CornerCrashDefine.MSG_WINDOW_HEIGHT);

            string dispmsg = disptxt.ToString();

            int start = 0;
            for (int i = 0; i < CornerCrashDefine.MSG_WINDOW_LINECNT; i++)
            {
                int nline = 0;

                if ((dispmsg.Length > start) == false)
                {
                    break;
                }

                nline = dispmsg.IndexOf("\n", start);

                if (start == nline)
                {
                    start++;
                    continue;
                }

                if (nline >= 0)
                {
                    msgtexture = fontrp.GetTexture(dispmsg.Substring(start, nline - start), 0);
                }
                else
                {
                    msgtexture = fontrp.GetTexture(dispmsg.Substring(start), 0);
                }

                screen.Blt(msgtexture, fontwidth,
                    (fontwidth / 2) + (int)((i * fontwidth) * 1.5) + CornerCrashDefine.WINDOW_HEIGHT - CornerCrashDefine.MSG_WINDOW_HEIGHT);

                msgtexture.Release();

                if (nline < 0)
                {
                    break;
                }

                start = nline + 1;
            }

            screen.Update();
        }

        private void Exec()
        {
            string chstr = "";
            int fontlength = (CornerCrashDefine.MSG_WINDOW_WIDTH / fontwidth);
            fontlength -= 2;

            isbreak = false;
            int i = curtxtpnt;
            
            string tagname;
            Dictionary<string, string> tagelm;

            for (; i < curSinario.sinario.Length; i++)
            {
                if ((linecnt >= CornerCrashDefine.MSG_WINDOW_LINECNT) ||
                    (exit == true) ||
                    (isbreak == true) )
                {
                    curtxtpnt = i;
                    return;
                }
                
                chstr = curSinario.sinario[i].ToString();
                switch (chstr)
                {
                    case "\n":
                        break;
                    case "[":
                        if (curSinario.sinario[i + 1].ToString() != "[")
                        {
                            i = ParseTag(i + 1, out tagname, out tagelm);                            
                            i = SinarioTag.Exec(i, tagname, tagelm, ref curSinario, ref sinariostack);
                        }
                        else
                        {
                            disptxt.Append("[");
                            fontcnt++;
                            i++;
                            isbreak = true;
                        }
                        break;
                    case ";":
                        if (fontcnt > 0)
                        {
                            disptxt.Append(curSinario.sinario[i]);
                            fontcnt++;
                            isbreak = true;
                            break;
                        }

                        if ( ((curSinario.sinario.Length - 1) > i) &&
                             (curSinario.sinario[i+1].ToString() == ";"))
                        {
                            disptxt.Append(curSinario.sinario[i]);
                            fontcnt++;
                            i++;
                            isbreak = true;
                            break;
                        }

                        if (curSinario.sinario.IndexOf("\n", i) >= 0)
                        {
                            i = curSinario.sinario.IndexOf("\n", i);
                        }
                        else
                        {
                            i = curSinario.sinario.Length - 2;
                        }
                        break;
                    default:
                        if (fontcnt >= fontlength)
                        {
                            disptxt.Append(curSinario.sinario[i]);
                            disptxt.Append("\n");
                            linecnt++;
                            fontcnt = 0;
                        }
                        else
                        {
                            disptxt.Append(curSinario.sinario[i]);
                            fontcnt++;
                            isbreak = true;
                        }
                        break;
                }
            }

            if (sinariostack.Count == 0)
            {
                enable = false;
                Exit();
                return;
            }

            curSinario = sinariostack.Pop();
            i = curSinario.pnt;

            curtxtpnt = i;
        }

        public bool IsExit
        {
            get
            {
                return exit;
            }
        }

        public bool ResumeReq
        {
            get
            {
                return resumeReq;
            }
        }

        public bool Enable
        {
            get
            {
                return enable;
            }
        }
    }
}
