using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.IO;
using Yanesdk.System;

namespace CornerCrash
{
    class Util
    {
        private static Util instance;

        private static void GenLoggerInstance()
        {
            if (instance == null)
            {
                instance = new Util();
            }
        }

        public static string LoadMapDataFromFile(string filename)
        {
            System.IO.StreamReader sr = new System.IO.StreamReader(filename);
            string strbuff;
            string str = "";

            while (null != (strbuff = sr.ReadLine()))
            {
                str += strbuff + "\n";
            }

            return str;
        }

        public static int[][] CnvMapDataFromStr(string mapdatastr)
        {
            string[] mapdatastrrows = mapdatastr.Split("\n".ToCharArray());

            int[][] mapdata = CnvMapDataFromStr(mapdatastrrows);

            return mapdata;
        }

        public static int[][] CnvMapDataFromStr(string[] mapdatastrrows)
        {
            int[][] mapdata = new int[mapdatastrrows.Length - 1][];
            for (int i = 0; i < (mapdatastrrows.Length - 1); i++)
            {
                string rowstr = mapdatastrrows[i];
                int[] mapdatarow = new int[rowstr.Length / 2];

                for (int j = 0; j < (mapdatarow.Length); j++)
                {
                    mapdatarow[j] = Convert.ToInt32(rowstr.Substring(j * 2, 2), 16);
                }
                mapdata[i] = mapdatarow;
            }

            return mapdata;
        }

        public static string DispStringBuild(string msg, int fontwidth)
        {
            int start = 0;
            int nline = 0;
            int fontlength = 0;
            fontlength = (CornerCrashDefine.MSG_WINDOW_WIDTH / fontwidth);
            fontlength -= 2;
            StringBuilder msgbuff = new StringBuilder();

            for (; ; )
            {
                nline = msg.IndexOf("\n", start);
                if ((nline - start) > fontlength)
                {
                    msgbuff.Append(msg.Substring(start, fontlength));
                    msgbuff.Append("\n");
                    start += fontlength;
                }
                else if (nline > 0)
                {
                    msgbuff.Append(msg.Substring(start, nline - start));
                    msgbuff.Append("\n");
                    start = nline + 1;
                }
                else if (nline == 0)
                {
                    start = nline + 1;
                }
                else if ((msg.Length - start) > fontlength)
                {
                    msgbuff.Append(msg.Substring(start, fontlength));
                    msgbuff.Append("\n");
                    start += fontlength;
                }
                else
                {
                    msgbuff.Append(msg.Substring(start));
                    break;
                }
            }

            if (msgbuff.Length == 0)
            {
                msgbuff.Append(msg);
            }

            return msgbuff.ToString();
        }

        public static string CnvByteArrayToUTF8Str(byte[] buff)
        {
            return Encoding.UTF8.GetString(buff);
        }

        public static string CnvByteArrayToStr(byte[] buff)
        {
            return Encoding.ASCII.GetString(buff);
        }

        public static Bitmap LoadBmp(string path)
        {
            GenLoggerInstance();
            
            if (Util.IsExist(path) == false)
            {
                throw new CornerCrashException(instance, System.Reflection.MethodBase.GetCurrentMethod(),
                    "t@C " + path + "܂B");
            }

            byte[] buff = null;

            if (Util.IsExist(path) == true)
            {
                buff = FileSys.Read(path);
            }

            return new Bitmap(new MemoryStream(buff));
        }

        public static Bitmap LoadBmp(Bitmap screen, Bitmap bmp, int x, int y)
        {
            Graphics g = Graphics.FromImage(screen);

            g.DrawImage(bmp, x, y, bmp.Width, bmp.Height);

            return screen;
        }
    
        public static Bitmap LoadBmp(Bitmap screen, Bitmap bmp, int x, int y, float a)
        {
            System.Drawing.Imaging.ColorMatrix cm = new System.Drawing.Imaging.ColorMatrix();
            cm.Matrix00 = 1;
            cm.Matrix11 = 1;
            cm.Matrix22 = 1;
            cm.Matrix33 = a;
            cm.Matrix44 = 1;

            //ImageAttributesIuWFNg̍쐬
            System.Drawing.Imaging.ImageAttributes ia =
                new System.Drawing.Imaging.ImageAttributes();
            //ColorMatrixݒ肷
            ia.SetColorMatrix(cm);
            Graphics g = Graphics.FromImage(screen);

            g.DrawImage(bmp,  new Rectangle(x, y, bmp.Width, bmp.Height), 0, 0, bmp.Width, bmp.Height,
                GraphicsUnit.Pixel, ia);

            return screen;
        }

        public static bool IsExist(string filename)
        {
            if (FileArchiverSingleton.getInstance().Archiver == null)
            {
                return FileSys.IsExist(filename);
            }

            else
            {
                return FileArchiverSingleton.getInstance().Archiver.IsExist(filename);
            }
        }
    }
}
